#pragma once
#ifndef Car_H
#define Car_H

#include <vector>
#include <memory>
#include <set>

#define EIGEN_DONT_ALIGN_STATICALLY
#include <Eigen/Dense>
#include <Eigen/Sparse>

#define GLM_FORCE_RADIANS
#include <glm/glm.hpp>
#include <glm/gtc/type_ptr.hpp>
#include <glm/gtx/string_cast.hpp>

#include "Shape.h"
#include "Wheel.h"

#include "Collider.h"

class Particle;
class Spring;
class MatrixStack;
class Program;

class Car
{
public:
	EIGEN_MAKE_ALIGNED_OPERATOR_NEW
	
	Car(int numX, int numY, int numZ,
		  const Eigen::Vector3d &center,
		  double width, double length, double height,
		  double mass,
		  double stiffness);
	virtual ~Car();
	
	void tare();
	void reset();
	void updatePosNor();
	void step(double h, const Eigen::Vector3d &grav, const std::vector< std::shared_ptr<Particle> > spheres, std::set<int> &pressedKeys, bool* keyToggles, std::vector< std::shared_ptr<Collider> > &colliders);
	
	void init();
	void draw(std::shared_ptr<MatrixStack> MV, const std::shared_ptr<Program> p) const;
	
	void load(const std::string &RESOURCE_DIR);
	
	glm::mat4 getMatrix();
	Vector3d lastCenter;
	
private:
	int numX;
	int numY;
	int numZ;
	int n;
	std::vector< std::shared_ptr<Particle> > particles;
	std::vector< std::shared_ptr<Spring> > springs;

	std::shared_ptr<Shape> wheelShape;
	
	Eigen::VectorXd v;
	Eigen::VectorXd f;
	
	std::vector<unsigned int> eleBuf;
	std::vector<float> posBuf;
	std::vector<float> norBuf;
	std::vector<float> texBuf;
	unsigned eleBufID;
	unsigned posBufID;
	unsigned norBufID;
	unsigned texBufID;

	bool drawMesh = true;
	bool drawBody = true;

	std::vector< std::shared_ptr<Wheel> > wheels;

	std::shared_ptr<Spring> createSpring(const std::shared_ptr<Particle> p0, const std::shared_ptr<Particle> p1, double E);
	void doSusp(int i, int j, int k, int wheelInd, std::set<int> &pressedKeys, std::vector< std::shared_ptr<Collider> > &colliders);
	void addFaceToElementBuf(int xStart, int xEnd, int yStart, int yEnd, int zStart, int zEnd, int xOffset, int yOffset, int zOffset);
};

#endif
