#pragma once

#define EIGEN_DONT_ALIGN_STATICALLY
#include <Eigen/Dense>

#define GLM_FORCE_RADIANS
#include <glm/glm.hpp>
#include <glm/gtc/type_ptr.hpp>
#include <glm/gtx/string_cast.hpp>

#include "Collider.h"
#include "Plane.h"

#include "Shape.h"
#include "MatrixStack.h"
#include "Program.h"

#ifndef M_PI
    #define M_PI 3.14159265358979323846
#endif

class Sphere : public Collider
{
public:
	EIGEN_MAKE_ALIGNED_OPERATOR_NEW

    Eigen::Vector3d position;
    double radius;

    std::shared_ptr<Shape> sphereShape;

    Sphere(Eigen::Vector3d pos, double _radius){
        position = pos;
        radius = _radius;
    }

    void update() {
    }

	double intersect(Eigen::Vector3d point, Eigen::Vector3d dir) {
        double det = pow(dir.dot(point - position), 2) - (pow((point - position).norm(), 2) - pow(radius, 2));
        if(det < 0)
            return -1;
        double d = -1 * dir.dot(point - position) - sqrt(det);
        return d;
    }

    void load(const std::string &RESOURCE_DIR) {
        sphereShape = std::make_shared<Shape>();
	    sphereShape->loadMesh(RESOURCE_DIR + "sphere2.obj");
    }

    void init() {
        sphereShape->init();
    }

    void draw(std::shared_ptr<MatrixStack> MV, const std::shared_ptr<Program> p) const {
        
        MV->pushMatrix();

        MV->translate(position(0), position(1), position(2));
        // MV->multMatrix(rotMat);

        MV->scale(radius);

        // MV->scale(2.0/30.0);
        // MV->rotate(M_PI/2, 0, 1, 0);
        // MV->translate(0, 0, 2.75);
        glUniformMatrix4fv(p->getUniform("MV"), 1, GL_FALSE, glm::value_ptr(MV->topMatrix()));
        // glUniform1f(p->getUniform("alpha"),  0.8);
        glUniform3fv(p->getUniform("kdFront"), 1, Eigen::Vector3f(0.5, 0.5, 0.5).data());
        // glUniform3fv(p->getUniform("kdBack"),  1, Eigen::Vector3f(0.1, 0.1, 0.1).data());
        sphereShape->draw(p);

        MV->popMatrix();

    }
};